@@ -54,6 +54,7 @@ import net.sourceforge.jnlp.util.BasicExceptionDialog;
 public class GuiLaunchHandler implements LaunchHandler {
 
     private JNLPSplashScreen splashScreen = null;
+    private final Object mutex = new Object();
     private UpdatePolicy policy = UpdatePolicy.ALWAYS;
 
     @Override
@@ -66,22 +67,29 @@ public class GuiLaunchHandler implements LaunchHandler {
         SwingUtilities.invokeLater(new Runnable() {
             @Override
             public void run() {
+                closeSplashScreen();
                 BasicExceptionDialog.show(exception);
             }
         });
     }
 
+    private void closeSplashScreen() {
+        synchronized(mutex) {
+            if (splashScreen != null) {
+                if (splashScreen.isSplashScreenValid()) {
+                    splashScreen.setVisible(false);
+                }
+                splashScreen.dispose();
+            }
+        }
+    }
+
     @Override
     public void launchStarting(ApplicationInstance application) {
         SwingUtilities.invokeLater(new Runnable() {
             @Override
             public void run() {
-                if (splashScreen != null) {
-                    if (splashScreen.isSplashScreenValid()) {
-                        splashScreen.setVisible(false);
-                    }
-                    splashScreen.dispose();
-                }
+                closeSplashScreen();
             }
         });
     }
@@ -89,8 +97,8 @@ public class GuiLaunchHandler implements LaunchHandler {
     @Override
     public void launchInitialized(final JNLPFile file) {
         
-        final int preferredWidth = 500;
-        final int preferredHeight = 400;
+        int preferredWidth = 500;
+        int preferredHeight = 400;
 
         final URL splashImageURL = file.getInformation().getIconLocation(
                 IconDesc.SPLASH, preferredWidth, preferredHeight);
@@ -98,30 +106,33 @@ public class GuiLaunchHandler implements LaunchHandler {
         if (splashImageURL != null) {
             final ResourceTracker resourceTracker = new ResourceTracker(true);
             resourceTracker.addResource(splashImageURL, file.getFileVersion(), null, policy);
+            synchronized(mutex) {
+                try {
+                    SwingUtilities.invokeAndWait(new Runnable() {
+                        @Override
+                        public void run() {
+                            splashScreen = new JNLPSplashScreen(resourceTracker, null, null);
+                        }
+                    });
+                } catch (InterruptedException ie) {
+                    // Wait till splash screen is created
+                    while (splashScreen == null);
+                } catch (InvocationTargetException ite) {
+                    ite.printStackTrace();
+                }
 
-            try {
-                SwingUtilities.invokeAndWait(new Runnable() {
-                    @Override
-                    public void run() {
-                        splashScreen = new JNLPSplashScreen(resourceTracker, null, null);
-                    }
-                });
-            } catch (InterruptedException ie) {
-                // Wait till splash screen is created
-                while (splashScreen == null);
-            } catch (InvocationTargetException ite) {
-                ite.printStackTrace();
+                splashScreen.setSplashImageURL(splashImageURL);
             }
-
-            splashScreen.setSplashImageURL(splashImageURL);
         }
-
+        
         SwingUtilities.invokeLater(new Runnable() {
             @Override
             public void run() {
                 if (splashImageURL != null) {
-                    if (splashScreen.isSplashScreenValid()) {
-                        splashScreen.setVisible(true);
+                    synchronized(mutex) {
+                        if (splashScreen.isSplashScreenValid()) {
+                            splashScreen.setVisible(true);
+                        }
                     }
                 }
             }
@@ -136,6 +147,7 @@ public class GuiLaunchHandler implements LaunchHandler {
 
     @Override
     public boolean validationError(LaunchException security) {
+        closeSplashScreen();
         DefaultLaunchHandler.printMessage(security);
         return true;
     }
