@@ -23,12 +23,15 @@
 package net.sourceforge.jnlp;
 
 import net.sourceforge.jnlp.AppletLog;
-import net.sourceforge.jnlp.runtime.AppThreadGroup;
 import net.sourceforge.jnlp.runtime.AppletInstance;
 import net.sourceforge.jnlp.runtime.JNLPRuntime;
 
 import java.net.URL;
+import java.util.HashMap;
 import java.util.Hashtable;
+import java.util.Map;
+import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.ConcurrentMap;
 
 import sun.applet.AppletViewerPanel;
 import sun.awt.SunToolkit;
@@ -44,9 +47,58 @@ public class NetxPanel extends AppletViewerPanel {
     private boolean exitOnFailure = true;
     private AppletInstance appInst = null;
     private boolean appletAlive;
+    private final String uKey;
+
+    // We use this so that we can create exactly one thread group
+    // for all panels with the same uKey.
+    private static final Map<String, ThreadGroup> uKeyToTG =
+        new HashMap<String, ThreadGroup>();
+    private static final Object TGMapMutex = new Object();
+
+    // This map is actually a set (unfortunately there is no ConcurrentSet
+    // in java.util.concurrent). If KEY is in this map, then we know that
+    // an app context has been created for the panel that has uKey.equals(KEY),
+    // so we avoid creating it a second time for panels with the same uKey.
+    // Because it's a set, only the keys matter. However, we can't insert
+    // null values in because if we did, we couldn't use null checks to see
+    // if a key was absent before a putIfAbsent. 
+    private static final ConcurrentMap<String, Boolean> appContextCreated =
+        new ConcurrentHashMap<String, Boolean>();
 
     public NetxPanel(URL documentURL, Hashtable<String, String> atts) {
         super(documentURL, atts);
+
+        /* According to http://download.oracle.com/javase/6/docs/technotes/guides/deployment/deployment-guide/applet-compatibility.html, 
+         * classloaders are shared iff these properties match:
+         * codebase, cache_archive, java_archive, archive
+         * 
+         * To achieve this, we create the uniquekey based on those 4 values,
+         * always in the same order. The initial "<NAME>=" parts ensure a 
+         * bad tag cannot trick the loader into getting shared with another.
+         */
+
+        String cache_archiveAttr = atts.get("cache_archive") != null ? 
+                                   atts.get("cache_archive") : "";
+
+        String java_archiveAttr =  atts.get("java_archive") != null ? 
+                                   atts.get("java_archive") : "";
+
+        String archiveAttr =       atts.get("archive") != null ? 
+                                   atts.get("archive") : "";
+
+        this.uKey = "codebase=" + getCodeBase().toExternalForm() +
+                    "cache_archive=" + cache_archiveAttr + 
+                    "java_archive=" + java_archiveAttr + 
+                    "archive=" +  archiveAttr;
+
+        // when this was being done (incorrectly) in Launcher, the call was
+        // new AppThreadGroup(mainGroup, file.getTitle());
+        synchronized(TGMapMutex) {
+            if (!uKeyToTG.containsKey(this.uKey)) {
+                ThreadGroup tg = new ThreadGroup(Launcher.mainGroup, this.documentURL.toString());
+                uKeyToTG.put(this.uKey, tg);
+            }
+        }
     }
 
     // overloaded constructor, called when initialized via plugin
@@ -57,18 +109,6 @@ public class NetxPanel extends AppletViewerPanel {
         this.appletAlive = true;
     }
 
-    @Override
-    public void run() {
-        /*
-         * create an AppContext for this thread associated with this particular
-         * plugin instance (which runs in a different thread group from the rest
-         * of the plugin).
-         */
-        SunToolkit.createNewAppContext();
-
-        super.run();
-    }
-
     @Override
     protected void showAppletException(Throwable t) {
         /*
@@ -78,7 +118,7 @@ public class NetxPanel extends AppletViewerPanel {
         AppletLog.log(t);
         super.showAppletException(t);
     }
-    
+
     //Overriding to use Netx classloader. You might need to relax visibility
     //in sun.applet.AppletPanel for runLoader().
     protected void runLoader() {
@@ -90,7 +130,7 @@ public class NetxPanel extends AppletViewerPanel {
                                 getCode(),
                                 getWidth(),
                                 getHeight(),
-                                atts);
+                                atts, uKey);
 
             doInit = true;
             dispatchAppletEvent(APPLET_LOADING, null);
@@ -118,8 +158,6 @@ public class NetxPanel extends AppletViewerPanel {
             // won't make it to the applet, whereas using sun.applet.AppletClassLoader
             // works just fine.
 
-            dispatchAppletEvent(APPLET_LOADING_COMPLETED, null);
-
             if (applet != null) {
                 // Stick it in the frame
                 applet.setStub(this);
@@ -131,7 +169,13 @@ public class NetxPanel extends AppletViewerPanel {
         } catch (Exception e) {
             this.appletAlive = false;
             e.printStackTrace();
-        }
+        } finally {
+            // PR1157: This needs to occur even in the case of an exception
+            // so that the applet's event listeners are signaled.
+            // Once PluginAppletViewer.AppletEventListener is signaled PluginAppletViewer it can properly stop waiting
+            // in PluginAppletViewer.waitForAppletInit
+            dispatchAppletEvent(APPLET_LOADING_COMPLETED, null);
+         }
     }
 
     /**
@@ -154,11 +198,7 @@ public class NetxPanel extends AppletViewerPanel {
             }
         }
 
-        // when this was being done (incorrectly) in Launcher, the call was
-        // new AppThreadGroup(mainGroup, file.getTitle());
-        ThreadGroup tg = new AppThreadGroup(Launcher.mainGroup,
-                this.documentURL.toString());
-        handler = new Thread(tg, this);
+        handler = new Thread(getThreadGroup(), this);
         handler.start();
     }
 
@@ -174,4 +214,21 @@ public class NetxPanel extends AppletViewerPanel {
     public boolean isAlive() {
         return handler != null && handler.isAlive() && this.appletAlive;
     }
+
+    public ThreadGroup getThreadGroup() {
+        synchronized(TGMapMutex) {
+            return uKeyToTG.get(uKey);
+        }
+    }
+
+    public void createNewAppContext() {
+        if (Thread.currentThread().getThreadGroup() != getThreadGroup()) {
+            throw new RuntimeException("createNewAppContext called from the wrong thread.");
+        }
+        // only create a new context if one hasn't already been created for the
+        // applets with this unique key.
+        if (null == appContextCreated.putIfAbsent(uKey, Boolean.TRUE)) {
+            SunToolkit.createNewAppContext();
+        }
+    }
 }
