@@ -316,7 +316,7 @@ IcedTeaPluginUtilities::strSplit(const char* str, const char* delim)
 		std::string* s = new std::string();
 		s->append(tok_ptr);
 		v->push_back(s);
-		tok_ptr = strtok (NULL, " ");
+		tok_ptr = strtok (NULL, delim);
 	}
 
 	return v;
@@ -668,11 +668,8 @@ IcedTeaPluginUtilities::printNPVariant(NPVariant variant)
     }
     else if (NPVARIANT_IS_STRING(variant))
     {
-#if MOZILLA_VERSION_COLLAPSED < 1090200
-    	PLUGIN_DEBUG("STRING: %s\n", NPVARIANT_TO_STRING(variant).utf8characters);
-#else
-    	PLUGIN_DEBUG("STRING: %s\n", NPVARIANT_TO_STRING(variant).UTF8Characters);
-#endif
+    	std::string str = IcedTeaPluginUtilities::NPVariantAsString(variant);
+    	PLUGIN_DEBUG("STRING: %s (length=%d)\n", str.c_str(), str.size());
     }
     else
     {
@@ -684,7 +681,7 @@ void
 IcedTeaPluginUtilities::NPVariantToString(NPVariant variant, std::string* result)
 {
 	char* str = (char*) malloc(sizeof(char)*32); // enough for everything except string
-
+    bool was_string_already = false;
     if (NPVARIANT_IS_VOID(variant))
     {
         sprintf(str, "%p", variant);
@@ -710,21 +707,15 @@ IcedTeaPluginUtilities::NPVariantToString(NPVariant variant, std::string* result
     }
     else if (NPVARIANT_IS_STRING(variant))
     {
-    	free(str);
-#if MOZILLA_VERSION_COLLAPSED < 1090200
-    	str = (char*) malloc(sizeof(char)*NPVARIANT_TO_STRING(variant).utf8length);
-    	sprintf(str, "%s", NPVARIANT_TO_STRING(variant).utf8characters);
-#else
-        str = (char*) malloc(sizeof(char)*NPVARIANT_TO_STRING(variant).UTF8Length);
-        sprintf(str, "%s", NPVARIANT_TO_STRING(variant).UTF8Characters);
-#endif
+    	result->append(IcedTeaPluginUtilities::NPVariantAsString(variant));
+    	was_string_already = true;
     }
     else
     {
         sprintf(str, "[Object %p]", variant);
     }
-
-    result->append(str);
+    if (!was_string_already)
+        result->append(str);
     free(str);
 }
 
@@ -864,13 +855,7 @@ IcedTeaPluginUtilities::isObjectJSArray(NPP instance, NPObject* object)
     browser_functions.invoke(instance, constructor, toString, NULL, 0, &constructor_str);
     IcedTeaPluginUtilities::printNPVariant(constructor_str);
 
-    std::string constructor_name = std::string();
-
-#if MOZILLA_VERSION_COLLAPSED < 1090200
-    constructor_name.append(NPVARIANT_TO_STRING(constructor_str).utf8characters);
-#else
-    constructor_name.append(NPVARIANT_TO_STRING(constructor_str).UTF8Characters);
-#endif
+    std::string constructor_name = IcedTeaPluginUtilities::NPVariantAsString(constructor_str);
 
     PLUGIN_DEBUG("Constructor for NPObject is %s\n", constructor_name.c_str());
 
@@ -913,6 +898,20 @@ IcedTeaPluginUtilities::decodeURL(const gchar* url, gchar** decoded_url)
     PLUGIN_DEBUG("SENDING URL: %s\n", *decoded_url);
 }
 
+/* Copies a variant data type into a C++ string */
+std::string
+IcedTeaPluginUtilities::NPVariantAsString(NPVariant variant)
+{
+#if MOZILLA_VERSION_COLLAPSED < 1090200
+  return std::string(
+    NPVARIANT_TO_STRING(variant).utf8characters,
+    NPVARIANT_TO_STRING(variant).utf8length);
+#else
+  return std::string(
+    NPVARIANT_TO_STRING(variant).UTF8Characters,
+    NPVARIANT_TO_STRING(variant).UTF8Length);
+#endif
+}
 
 /**
  * Posts a function for execution on the plug-in thread and wait for result.
